@@ -8,6 +8,9 @@ import sys
 import os
 import math
 
+def angle_point(x0,y0,a,radius):
+    return (int(x0+(1.5*math.cos(math.radians(a))*radius)), int(y0+math.sin(math.radians(a))*radius))
+
 class Canvas:
     """ primitive drawing surface attached to a curses window """
 
@@ -141,6 +144,10 @@ class Canvas:
             self.max_y,self.max_x = (0,0)
             self.char_map = None
 
+    def clear( self ):
+        """ clear the entire canvas """
+        self.char_map = [[None] * self.max_y for i in range(self.max_x)]
+
     def refresh( self ):
         """ refresh the display after drawing """
         self.win.refresh()
@@ -154,6 +161,9 @@ class Canvas:
         if x < 0 or x >= self.max_x or y < 0 or y >= self.max_y:
             return
         row,col = self.to_rowcol(x,y)
+        if row >= self.max_y//2 or col >= self.max_x//2:
+            return
+
         mask = self.to_mask[(int(x)%2)+((int(y)%2)*2)]
 
         if not self.char_map[col][row]:
@@ -172,23 +182,20 @@ class Canvas:
 
     def line(self, x0, y0, x1, y1, color, put_pixel=None ):
         """ draw a line between x0,y0 and x1,y1 in color """
-        x0 = int(x0)
-        x1 = int(x1)
-        y0 = int(y0)
-        y1 = int(y1)
-
+        def set_pixel(x,y,color):
+            if put_pixel:
+                put_pixel(x,y,color)
+            else:
+                self.put_pixel(x,y,color)
         dx = abs(x1 - x0)
         dy = abs(y1 - y0)
-        x, y = x0, y0
+        x, y = int(x0), int(y0)
         sx = -1 if x0 > x1 else 1
         sy = -1 if y0 > y1 else 1
         if dx > dy:
             err = dx / 2.0
-            while x != x1:
-                if put_pixel:
-                    put_pixel(x,y,color)
-                else:
-                    self.put_pixel(x, y, color)
+            while x != int(x1):
+                set_pixel(x, y,color)
                 err -= dy
                 if err < 0:
                     y += sy
@@ -196,20 +203,14 @@ class Canvas:
                 x += sx
         else:
             err = dy / 2.0
-            while y != y1:
-                if put_pixel:
-                    put_pixel(x,y,color)
-                else:
-                    self.put_pixel(x, y, color)
+            while y != int(y1):
+                set_pixel(x, y,color)
                 err -= dx
                 if err < 0:
                     x += sx
                     err += dy
                 y += sy
-        if put_pixel:
-            put_pixel(x, y, color)
-        else:
-            self.put_pixel(x,y,color)
+        set_pixel(x, y, color)
 
     def intersect( self, seg1, seg2, clip_to_seg = False ):
         """ find the intersection of two segments as tuples (x0,y0,x1,y1) returns tuple (x,y) if no intersection returns None """
@@ -340,50 +341,143 @@ class Canvas:
     def rasterize( self, points, color, put_pixel=None):
         """ sort points representing the boundary of a filled shape and rasterize by filling lines with color """
         ps = sorted(points,key=lambda x: (x[1],x[0]))
-        sp = iter(ps)
-        p2 = p1 = next(sp,None)
-        si = 0
-        last = None
-        while p2:
-            last = p2
-            p2 = next(sp,None)
-            if not p2:
-                if put_pixel:
-                    put_pixel(p1[0],p1[1],color)
-                else:
-                    self.put_pixel(p1[0],p1[1],color)
+        n_points = len(ps)
+        if n_points == 0:
+            return
+        elif n_points == 1:
+            x,y = ps[0]
+            if put_pixel:
+                put_pixel(x,y,color)
             else:
-                if p2[1] == p1[1]:
-                    continue
+                self.put_pixel(x,y,color)
+        else:
+            idx = 1
+            x0,y0 = ps[0]
+            x1,y1 = x0,y0
+            while idx < len(ps):
+                xn,yn = ps[idx]
+                if yn == y0:
+                    x1,y1 = xn,yn
                 else:
-                    self.line(p1[0],p1[1],last[0],last[1],color,put_pixel)
-                    p1 = p2
-
-
+                    if x0 == x1:
+                        if put_pixel:
+                            put_pixel(x0,y0,color)
+                        else:
+                            self.put_pixel(x0,y0,color)
+                    else:
+                        self.line(x0,y0,x1,y1,color,put_pixel)
+                    x0,y0 = xn,yn
+                    x1,y1 = x0,y0
+                idx += 1
+            if x0 == x1:
+                if put_pixel:
+                    put_pixel(x0,y0,color)
+                else:
+                    self.put_pixel(x0,y0,color)
+            else:
+                self.line(x0,y0,x1,y1,color,put_pixel)
 
     def circle(self, x0, y0, radius, color, fill = False, put_pixel=None ):
         """ draw a circle centered at x0,y0 of radius radius in color """
-        self.arc(x0,y0,radius,0,360,color,fill,put_pixel)
+        points = []
+
+        def circle_point( points, xc,yc,x,y ):
+            points.extend([(xc+x, yc+y),(xc-x, yc+y),(xc+x, yc-y),(xc-x, yc-y),(xc+y, yc+x),(xc-y, yc+x),(xc+y, yc-x),(xc-y, yc-x)])
+
+        x0 = int(x0)
+        y0 = int(y0)
+        radius = int(radius)
+        x = 0
+        y = radius
+        d = 3 - 2 * radius
+        circle_point(points,x0,y0,x,y)
+        while y >= x:
+            x += 1
+            if d > 0:
+                y -= 1
+                d = d + 4 * (x - y) + 10
+            else:
+                d = d + 4 * x + 6
+            circle_point(points,x0,y0,x,y)
+
+        for idx in range(len(points)):
+            x,y = points[idx]
+            x = int(((x-x0)*1.5)+x0)
+            points[idx] = (x,y)
+
+        if not fill:
+            for x,y in points:
+                if put_pixel:
+                    put_pixel(x,y,color)
+                else:
+                    self.put_pixel(x,y,color)
+        else:
+            self.rasterize(points,color,put_pixel)
 
     def arc(self,x0,y0,radius,a0,a1,color,fill=False,put_pixel=None,just_points=False):
         """ draw an arc between a0 degrees to a1 degrees centered at x0,y0 with radius and color """
-        def circle_point(x0,y0,a,radius):
-            return (x0+(1.5*math.cos(math.radians(a))*radius), y0+math.sin(math.radians(a))*radius)
-
         points = []
-        points.append((x0,y0))
-        a = a0
-        while a <= a1:
-            xp,yp = circle_point(x0,y0,a,radius)
-            a = a + 1.0
-            points.append((xp,yp))
-                                
+
+        def circle_point( points, xc,yc,x,y ):
+            points.extend([(xc+x, yc+y),(xc-x, yc+y),(xc+x, yc-y),(xc-x, yc-y),(xc+y, yc+x),(xc-y, yc+x),(xc+y, yc-x),(xc-y, yc-x)])
+
+        x0 = int(x0)
+        y0 = int(y0)
+        radius = int(radius)
+        x = 0
+        y = radius
+        d = 3 - 2 * radius
+        circle_point(points,x0,y0,x,y)
+        while y >= x:
+            x += 1
+            if d > 0:
+                y -= 1
+                d = d + 4 * (x - y) + 10
+            else:
+                d = d + 4 * x + 6
+            circle_point(points,x0,y0,x,y)
+
+        xs,ys = angle_point(x0,y0,a0,radius+0.5)
+        xe,ye = angle_point(x0,y0,a1,radius+0.5)
+        xm,ym = angle_point(x0,y0,(a0+a1)/2,radius+0.5)
+        x_min = min(x0,xs,xe,xm)
+        y_min = min(y0,ys,ye,ym)
+        x_max = max(x0,xs,xe,xm)
+        y_max = max(y0,ys,ye,ym)
+
+        # for drawing the previous one was for bounding
+        xs,ys = angle_point(x0,y0,a0,radius)
+        xe,ye = angle_point(x0,y0,a1,radius)
+
+        filtered_points = []
+        for x,y in points:
+            px = int(((x-x0)*1.5)+x0)
+            if px >= x_min and px <= x_max and y >= y_min and y <= y_max:
+                angle = math.degrees(math.atan2(y-y0,x-x0))
+                if angle < 0:
+                    angle += 360
+                if angle >= a0 and angle <= a1:
+                    filtered_points.append((px,y))
+        points = filtered_points
 
         if just_points:
             for x,y in points:
                 put_pixel(x,y,color)
         else:
-            self.polygon(points,color,fill,put_pixel)
+            if not fill:
+                self.line(x0,y0,xs,ys,color,put_pixel)
+                self.line(x0,y0,xe,ye,color,put_pixel)
+                for x,y in points:
+                    if put_pixel:
+                        put_pixel(x,y,color)
+                    else:
+                        self.put_pixel(x,y,color)
+            else:
+                def add_pixel( x,y,color ):
+                    points.append((x,y))
+                self.line(x0,y0,xs,ys,color,add_pixel)
+                self.line(x0,y0,xe,ye,color,add_pixel)
+                self.rasterize(points,color,put_pixel)
 
     def rect(self,x0,y0,x1,y1,color,fill=False,put_pixel=None):
         """ draw a rectangle bounding x0,y0, x1,y1, in color == color optionally filling """
@@ -429,18 +523,20 @@ class Canvas:
 
     def polyline(self,points,color,put_pixel=None):
         """ draw a polyline defined by the sequence points which represent a list of (x,y) tuples in the order they should be connected in color """
-        i = iter(points)
-        p1 = next(i,None)
-        while p1:
-            p2 = next(i,None)
-            if p2:
-                self.line(p1[0],p1[1],p2[0],p2[1],color,put_pixel)
+        n_points = len(points)
+        if n_points == 0:
+            return
+        elif n_points == 1:
+            x,y = points[0]
+            if put_pixel:
+                put_pixel(x,y,color)
             else:
-                if put_pixel:
-                    put_pixel(p1[0],p1[1],color)
-                else:
-                    self.put_pixel(p1[0],p1[1],color)
-            p1 = p2
+                self.put_pixel(x,y,color)
+        else:
+            for idx in range(n_points-1):
+                x0,y0 = points[idx]
+                x1,y1 = points[idx+1]
+                self.line(x0,y0,x1,y1,color,put_pixel)
 
     def poly_fill(self,points,color,put_pixel = None):
         """ fill a concave polygon by recursively subdividing until we get a convex polygon """
